/*
 * gps_boot_2.c
 *
 * Created: 03.06.2014 17:50:58
 *  Author: bl
 */ 

//Define F_CPU
#define F_CPU 18432000UL

//includes
#include <avr/io.h>
#include <stdlib.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <string.h>

//externe includes
#include "lcd-routines.h"	// aus Mikrocontoller.net
#include "uart.h"			// Peter Fleury UART Lib

//Defines
#define UART0_BAUD_RATE 4800
#define UART1_BAUD_RATE 57600
#define buffersize 85

#define debounce( port, pin )                                         \
({                                                                    \
	static uint8_t flag = 0;     /* new variable on every macro usage */  \
	uint8_t i = 0;                                                      \
	\
	if( flag ){                  /* check for key release: */           \
		for(;;){                   /* loop ... */                         \
			if( !(port & 1<<pin) ){  /* ... until key pressed or ... */     \
				i = 0;                 /* 0 = bounce */                       \
				break;                                                        \
			}                                                               \
			_delay_us( 98 );         /* * 256 = 25ms */                     \
			if( --i == 0 ){          /* ... until key >25ms released */     \
				flag = 0;              /* clear press flag */                 \
				i = 0;                 /* 0 = key release debounced */        \
				break;                                                        \
			}                                                               \
		}                                                                 \
		}else{                       /* else check for key press: */        \
		for(;;){                   /* loop ... */                         \
			if( (port & 1<<pin) ){   /* ... until key released or ... */    \
				i = 0;                 /* 0 = bounce */                       \
				break;                                                        \
			}                                                               \
			_delay_us( 98 );         /* * 256 = 25ms */                     \
			if( --i == 0 ){          /* ... until key >25ms pressed */      \
				flag = 1;              /* set press flag */                   \
				i = 1;                 /* 1 = key press debounced */          \
				break;                                                        \
			}                                                               \
		}                                                                 \
	}                                                                   \
	i;                           /* return value of Macro */            \
})


//Variablen
unsigned char state = 1;
static char gpsstring[buffersize];
volatile uint8_t i = 0;


void stateMachine()
{
	switch( state )	{
		case 1:
		lcd_clear();
		lcd_setcursor( 0, 1 );
		lcd_string("Menu 1:");
		lcd_setcursor( 0, 2 );
		lcd_string("Test1");
		state = 2;
		break;
		
		case 2:
		lcd_clear();
		lcd_setcursor( 0, 1 );
		lcd_string("Menu 2:");
		lcd_setcursor( 0, 2 );
		lcd_string("Test2");
		state = 3;
		break;
		
		case 3:
		lcd_clear();
		lcd_setcursor( 0, 1 );
		lcd_string("Menu 3:");
		lcd_setcursor( 0, 2 );
		lcd_string("Test3");
		state = 4;
		break;
		
		case 4:
		lcd_clear();
		lcd_setcursor( 0, 1 );
		lcd_string("Menu 4:");
		lcd_setcursor( 0, 2 );
		lcd_string("Test5");
		state = 5;
		break;
		
		case 5:
		lcd_clear();
		lcd_setcursor( 0, 1 );
		lcd_string("Menu 5:");
		lcd_setcursor( 0, 2 );
		lcd_string("Test6");
		state = 1;
		break;
		
	}
}

int main(void)
{
	
	lcd_init();
	
	stateMachine();
	
	// Ports initialisieren
	DDRA &= ~(1<<PA7);		//PA7 auf Eingang
	PORTA |= 1<<PA7;		//PA7 Pullup aktivieren
	DDRC |= 1<<PC7;			//PC7 auf Ausgang
	DDRA |= 1<<PA6;			//PA6 auf Ausgang
	
	PORTA |= 1<<PA6;		//PA6 auf HIGH
	// PORTC |= 1<<PC7;		//PC7 auf HIGH
	
	// UARTs initialisieren - UART0= GPS; UART1= extern
	uart_init( UART_BAUD_SELECT(UART0_BAUD_RATE,F_CPU) );
	uart1_init( UART_BAUD_SELECT(UART1_BAUD_RATE,F_CPU) );
	
	sei();
	
    for(;;)
    {
        if( debounce( PINA, PA7 ) )
			stateMachine();
		
		for (i = 0; i < buffersize; i++)
		{
			gpsstring[i] = uart_getc();
			
			if ( gpsstring[i] & UART_NO_DATA )
			{
				/* 
				* no data available from UART 
				*/
			}		
			else
				{
					/*
					* new data available from UART
					* check for Frame or Overrun error
					*/
				if ( gpsstring[i] & UART_FRAME_ERROR )
				{
					/* Framing Error detected, i.e no stop bit detected */
					uart1_puts_P("UART Frame Error: ");
				}
				if ( gpsstring[i] & UART_OVERRUN_ERROR )
				{
					/* 
					* Overrun, a character already present in the UART UDR register was 
					* not read by the interrupt handler before the next character arrived,
					* one or more received characters have been dropped
					*/
					uart1_puts_P("UART Overrun Error: ");
				}
				if ( gpsstring[i] & UART_BUFFER_OVERFLOW )
				{
					/* 
					* We are not reading the receive buffer fast enough,
					* one or more received character have been dropped 
					*/
					uart1_puts_P("Buffer overflow error: ");
				}
            
				uart1_putc( (unsigned char)gpsstring[i] );	//testweise den aktuellen Stream ausgeben
				
				if (gpsstring[i] == '\n')
				{
					gpsstring[i+1] = '\0';
					i = buffersize;
					
					// uart1_putc( (unsigned char)gpsstring[i] ); //testweise den aktuellen Stream ausgeben
				
					if (gpsstring[0] == '$' && gpsstring[1] == 'G' &&
						gpsstring[2] == 'P' && gpsstring[3] == 'G' &&
						gpsstring[4] == 'G' && gpsstring[5] == 'A')
					{
						uart1_puts("GPGGA String gefunden \r\n");
					
					}
						
					if (gpsstring[0] == '$' && gpsstring[1] == 'G' &&
						gpsstring[2] == 'P' && gpsstring[3] == 'R' &&
						gpsstring[4] == 'M' && gpsstring[5] == 'C')	
					{
						uart1_puts("GPRMC String gefunden \r\n");
					
					}
				}
			}
		}
	}
}